@@ -296,6 +296,22 @@ public class ScheduledDeliveryHandlerTest extends Assert {
 
    class FakeMessage implements Message {
 
+      @Override
+      public Object getUserContext(Object key) {
+         return null;
+      }
+
+      @Override
+      public void setUserContext(Object key, Object value) {
+
+      }
+
+      @Override
+      public String getProtocolName() {
+         // should normally not be visible in GUI
+         return getClass().getName();
+      }
+
       @Override
       public SimpleString getReplyTo() {
          return null;
@@ -1197,7 +1213,7 @@ public class ScheduledDeliveryHandlerTest extends Assert {
       }
 
       @Override
-      public void acknowledge(Transaction tx, MessageReference ref, AckReason reason, ServerConsumer consumer) throws Exception {
+      public void acknowledge(Transaction tx, MessageReference ref, AckReason reason, ServerConsumer consumer, boolean delivering) throws Exception {
 
       }
 
@@ -1402,7 +1418,7 @@ public class ScheduledDeliveryHandlerTest extends Assert {
       }
 
       @Override
-      public void expire(MessageReference ref, ServerConsumer consumer) throws Exception {
+      public void expire(MessageReference ref, ServerConsumer consumer, boolean delivering) throws Exception {
 
       }
 
@@ -1517,6 +1533,11 @@ public class ScheduledDeliveryHandlerTest extends Assert {
          return null;
       }
 
+      @Override
+      public SimpleString getDeadLetterAddress() {
+         return null;
+      }
+
       @Override
       public void pause() {
 
@@ -1612,6 +1633,11 @@ public class ScheduledDeliveryHandlerTest extends Assert {
 
       }
 
+      @Override
+      public void postAcknowledge(MessageReference ref, AckReason reason, boolean delivering) {
+
+      }
+
       @Override
       public void postAcknowledge(MessageReference ref, AckReason reason) {
 
