@@ -25,28 +25,30 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
-import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
+import java.util.concurrent.ScheduledThreadPoolExecutor;
 import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicLong;
 
-import org.apache.activemq.transport.InactivityIOException;
+import org.apache.activemq.transport.netty.NettyTransport;
 import org.apache.activemq.transport.amqp.client.sasl.SaslAuthenticator;
-import org.apache.activemq.transport.amqp.client.transport.NettyTransportListener;
+import org.apache.activemq.transport.netty.NettyTransportListener;
 import org.apache.activemq.transport.amqp.client.util.AsyncResult;
 import org.apache.activemq.transport.amqp.client.util.ClientFuture;
 import org.apache.activemq.transport.amqp.client.util.IdGenerator;
 import org.apache.activemq.transport.amqp.client.util.NoOpAsyncResult;
-import org.apache.activemq.transport.amqp.client.util.UnmodifiableConnection;
+import org.apache.activemq.transport.amqp.client.util.UnmodifiableProxy;
 import org.apache.qpid.proton.amqp.Symbol;
 import org.apache.qpid.proton.engine.Collector;
 import org.apache.qpid.proton.engine.Connection;
+import org.apache.qpid.proton.engine.Delivery;
 import org.apache.qpid.proton.engine.EndpointState;
 import org.apache.qpid.proton.engine.Event;
 import org.apache.qpid.proton.engine.Event.Type;
 import org.apache.qpid.proton.engine.Sasl;
+import org.apache.qpid.proton.engine.Session;
 import org.apache.qpid.proton.engine.Transport;
 import org.apache.qpid.proton.engine.impl.CollectorImpl;
 import org.apache.qpid.proton.engine.impl.TransportImpl;
@@ -54,7 +56,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import io.netty.buffer.ByteBuf;
-import io.netty.buffer.Unpooled;
 import io.netty.util.ReferenceCountUtil;
 
 public class AmqpConnection extends AmqpAbstractResource<Connection> implements NettyTransportListener {
@@ -73,19 +74,20 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
    public static final long DEFAULT_CLOSE_TIMEOUT = 30000;
    public static final long DEFAULT_DRAIN_TIMEOUT = 60000;
 
-   private final ScheduledExecutorService serializer;
+   private ScheduledThreadPoolExecutor serializer;
    private final AtomicBoolean closed = new AtomicBoolean();
    private final AtomicBoolean connected = new AtomicBoolean();
    private final AtomicLong sessionIdGenerator = new AtomicLong();
    private final AtomicLong txIdGenerator = new AtomicLong();
    private final Collector protonCollector = new CollectorImpl();
-   private final org.apache.activemq.transport.amqp.client.transport.NettyTransport transport;
+   private final NettyTransport transport;
    private final Transport protonTransport = Transport.Factory.create();
 
    private final String username;
    private final String password;
    private final URI remoteURI;
    private final String connectionId;
+   private List<Symbol> desiredCapabilities = Collections.emptyList();
    private List<Symbol> offeredCapabilities = Collections.emptyList();
    private Map<Symbol, Object> offeredProperties = Collections.emptyMap();
 
@@ -100,16 +102,16 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
    private boolean idleProcessingDisabled;
    private String containerId;
    private boolean authenticated;
+   private int maxFrameSize = DEFAULT_MAX_FRAME_SIZE;
    private int channelMax = DEFAULT_CHANNEL_MAX;
+   private int sessionIncomingCapacity = 0;
    private long connectTimeout = DEFAULT_CONNECT_TIMEOUT;
    private long closeTimeout = DEFAULT_CLOSE_TIMEOUT;
    private long drainTimeout = DEFAULT_DRAIN_TIMEOUT;
    private boolean trace;
    private boolean noContainerID = false;
 
-   public AmqpConnection(org.apache.activemq.transport.amqp.client.transport.NettyTransport transport,
-                         String username,
-                         String password) {
+   public AmqpConnection(NettyTransport transport, String username, String password) {
       setEndpoint(Connection.Factory.create());
       getEndpoint().collect(protonCollector);
 
@@ -119,7 +121,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
       this.connectionId = CONNECTION_ID_GENERATOR.generateId();
       this.remoteURI = transport.getRemoteLocation();
 
-      this.serializer = Executors.newSingleThreadScheduledExecutor(new ThreadFactory() {
+      this.serializer = new ScheduledThreadPoolExecutor(1, new ThreadFactory() {
 
          @Override
          public Thread newThread(Runnable runner) {
@@ -130,7 +132,12 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
          }
       });
 
+      // Ensure timely shutdown
+      this.serializer.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
+      this.serializer.setContinueExistingPeriodicTasksAfterShutdownPolicy(false);
+
       this.transport.setTransportListener(this);
+      this.transport.setMaxFrameSize(getMaxFrameSize());
    }
 
    public void connect() throws Exception {
@@ -145,6 +152,9 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
                   getEndpoint().setContainer(safeGetContainerId());
                }
                getEndpoint().setHostname(remoteURI.getHost());
+               if (!getDesiredCapabilities().isEmpty()) {
+                  getEndpoint().setDesiredCapabilities(getDesiredCapabilities().toArray(new Symbol[0]));
+               }
                if (!getOfferedCapabilities().isEmpty()) {
                   getEndpoint().setOfferedCapabilities(getOfferedCapabilities().toArray(new Symbol[0]));
                }
@@ -157,6 +167,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
                }
                protonTransport.setMaxFrameSize(getMaxFrameSize());
                protonTransport.setChannelMax(getChannelMax());
+               protonTransport.setEmitFlowEventOnSend(false);
                protonTransport.bind(getEndpoint());
                Sasl sasl = protonTransport.sasl();
                if (sasl != null) {
@@ -268,7 +279,9 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
          @Override
          public void run() {
             checkClosed();
-            session.setEndpoint(getEndpoint().session());
+            Session protonSession = getEndpoint().session();
+            protonSession.setIncomingCapacity(sessionIncomingCapacity);
+            session.setEndpoint(protonSession);
             session.setStateInspector(getStateInspector());
             session.open(request);
             pumpToProtonTransport(request);
@@ -280,31 +293,6 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
       return session;
    }
 
-   //----- Access to low level IO for specific test cases -------------------//
-
-   public void sendRawBytes(final byte[] rawData) throws Exception {
-      checkClosed();
-
-      final ClientFuture request = new ClientFuture();
-
-      serializer.execute(new Runnable() {
-
-         @Override
-         public void run() {
-            checkClosed();
-            try {
-               transport.send(Unpooled.wrappedBuffer(rawData));
-            } catch (IOException e) {
-               fireClientException(e);
-            } finally {
-               request.onSuccess();
-            }
-         }
-      });
-
-      request.sync();
-   }
-
    //----- Configuration accessors ------------------------------------------//
 
    /**
@@ -357,7 +345,11 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
     * @return the currently set Max Frame Size value.
     */
    public int getMaxFrameSize() {
-      return DEFAULT_MAX_FRAME_SIZE;
+      return maxFrameSize;
+   }
+
+   public void setMaxFrameSize(int maxFrameSize) {
+      this.maxFrameSize = maxFrameSize;
    }
 
    public int getChannelMax() {
@@ -368,6 +360,14 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
       this.channelMax = channelMax;
    }
 
+   public int getSessionIncomingCapacity() {
+      return sessionIncomingCapacity;
+   }
+
+   public void setSessionIncomingCapacity(int capacity) {
+      this.sessionIncomingCapacity = capacity;
+   }
+
    public long getConnectTimeout() {
       return connectTimeout;
    }
@@ -392,12 +392,24 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
       this.drainTimeout = drainTimeout;
    }
 
+   public List<Symbol> getDesiredCapabilities() {
+      return desiredCapabilities;
+   }
+
+   public void setDesiredCapabilities(List<Symbol> desiredCapabilities) {
+      if (desiredCapabilities == null) {
+         desiredCapabilities = Collections.emptyList();
+      }
+
+      this.desiredCapabilities = desiredCapabilities;
+   }
+
    public List<Symbol> getOfferedCapabilities() {
       return offeredCapabilities;
    }
 
    public void setOfferedCapabilities(List<Symbol> offeredCapabilities) {
-      if (offeredCapabilities != null) {
+      if (offeredCapabilities == null) {
          offeredCapabilities = Collections.emptyList();
       }
 
@@ -409,7 +421,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
    }
 
    public void setOfferedProperties(Map<Symbol, Object> offeredProperties) {
-      if (offeredProperties != null) {
+      if (offeredProperties == null) {
          offeredProperties = Collections.emptyMap();
       }
 
@@ -417,7 +429,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
    }
 
    public Connection getConnection() {
-      return new UnmodifiableConnection(getEndpoint());
+      return UnmodifiableProxy.connectionProxy(getEndpoint());
    }
 
    public AmqpConnectionListener getListener() {
@@ -511,7 +523,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
             if (toWrite != null && toWrite.hasRemaining()) {
                ByteBuf outbound = transport.allocateSendBuffer(toWrite.remaining());
                outbound.writeBytes(toWrite);
-               transport.send(outbound);
+               transport.sendVoidPromise(outbound);
                protonTransport.outputConsumed();
             } else {
                done = true;
@@ -551,7 +563,8 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
                buffer.put(duplicate);
                protonTransport.processInput();
                source.position(source.position() + limit);
-            } while (source.hasRemaining());
+            }
+            while (source.hasRemaining());
 
             ReferenceCountUtil.release(incoming);
 
@@ -585,7 +598,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
             // Using nano time since it is not related to the wall clock, which may change
             long initialNow = TimeUnit.NANOSECONDS.toMillis(System.nanoTime());
             long initialKeepAliveDeadline = protonTransport.tick(initialNow);
-            if (initialKeepAliveDeadline > 0) {
+            if (initialKeepAliveDeadline != 0) {
 
                getScheduler().schedule(new Runnable() {
 
@@ -596,15 +609,15 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
                            LOG.debug("Client performing next idle check");
                            // Using nano time since it is not related to the wall clock, which may change
                            long now = TimeUnit.NANOSECONDS.toMillis(System.nanoTime());
-                           long rescheduleAt = protonTransport.tick(now) - now;
+                           long deadline = protonTransport.tick(now);
                            pumpToProtonTransport();
                            if (protonTransport.isClosed()) {
                               LOG.debug("Transport closed after inactivity check.");
-                              throw new InactivityIOException("Channel was inactive for to long");
-                           }
-
-                           if (rescheduleAt > 0) {
-                              getScheduler().schedule(this, rescheduleAt, TimeUnit.MILLISECONDS);
+                              throw new IllegalStateException("Channel was inactive for too long");
+                           } else {
+                              if (deadline != 0) {
+                                 getScheduler().schedule(this, deadline - now, TimeUnit.MILLISECONDS);
+                              }
                            }
                         }
                      } catch (Exception e) {
@@ -697,7 +710,7 @@ public class AmqpConnection extends AmqpAbstractResource<Connection> implements
                   break;
                case DELIVERY:
                   amqpEventSink = (AmqpEventSink) protonEvent.getLink().getContext();
-                  amqpEventSink.processDeliveryUpdates(this);
+                  amqpEventSink.processDeliveryUpdates(this, (Delivery) protonEvent.getContext());
                   break;
                default:
                   break;
