@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -35,8 +35,7 @@ import javax.jms.InvalidDestinationException;
 import org.apache.activemq.transport.amqp.client.util.AsyncResult;
 import org.apache.activemq.transport.amqp.client.util.ClientFuture;
 import org.apache.activemq.transport.amqp.client.util.IOExceptionSupport;
-import org.apache.activemq.transport.amqp.client.util.UnmodifiableReceiver;
-import org.apache.qpid.jms.JmsOperationTimedOutException;
+import org.apache.activemq.transport.amqp.client.util.UnmodifiableProxy;
 import org.apache.qpid.proton.amqp.Binary;
 import org.apache.qpid.proton.amqp.DescribedType;
 import org.apache.qpid.proton.amqp.Symbol;
@@ -70,12 +69,16 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    private final AmqpSession session;
    private final String address;
    private final String receiverId;
+
    private final Source userSpecifiedSource;
+   private final SenderSettleMode userSpecifiedSenderSettlementMode;
+   private final ReceiverSettleMode userSpecifiedReceiverSettlementMode;
 
    private String subscriptionName;
    private String selector;
    private boolean presettle;
    private boolean noLocal;
+   private Map<Symbol, Object> properties;
 
    private AsyncResult pullRequest;
    private AsyncResult stopRequest;
@@ -83,11 +86,32 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Create a new receiver instance.
     *
-    * @param session    The parent session that created the receiver.
-    * @param address    The address that this receiver should listen on.
-    * @param receiverId The unique ID assigned to this receiver.
+    * @param session
+    *        The parent session that created the receiver.
+    * @param address
+    *        The address that this receiver should listen on.
+    * @param receiverId
+    *        The unique ID assigned to this receiver.
     */
    public AmqpReceiver(AmqpSession session, String address, String receiverId) {
+      this(session, address, receiverId, null, null);
+   }
+
+   /**
+    * Create a new receiver instance.
+    *
+    * @param session
+    *        The parent session that created the receiver.
+    * @param address
+    *        The address that this receiver should listen on.
+    * @param receiverId
+    *        The unique ID assigned to this receiver.
+    * @param senderMode
+    *        The {@link SenderSettleMode} to use on open.
+    * @param receiverMode
+    *        The {@link ReceiverSettleMode} to use on open.
+    */
+   public AmqpReceiver(AmqpSession session, String address, String receiverId, SenderSettleMode senderMode, ReceiverSettleMode receiverMode) {
 
       if (address != null && address.isEmpty()) {
          throw new IllegalArgumentException("Address cannot be empty.");
@@ -97,14 +121,19 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       this.session = session;
       this.address = address;
       this.receiverId = receiverId;
+      this.userSpecifiedSenderSettlementMode = senderMode;
+      this.userSpecifiedReceiverSettlementMode = receiverMode;
    }
 
    /**
     * Create a new receiver instance.
     *
-    * @param session    The parent session that created the receiver.
-    * @param source     The Source instance to use instead of creating and configuring one.
-    * @param receiverId The unique ID assigned to this receiver.
+    * @param session
+    *        The parent session that created the receiver.
+    * @param source
+    *        The Source instance to use instead of creating and configuring one.
+    * @param receiverId
+    *        The unique ID assigned to this receiver.
     */
    public AmqpReceiver(AmqpSession session, Source source, String receiverId) {
 
@@ -113,16 +142,19 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       }
 
       this.session = session;
-      this.userSpecifiedSource = source;
       this.address = source.getAddress();
       this.receiverId = receiverId;
+      this.userSpecifiedSource = source;
+      this.userSpecifiedSenderSettlementMode = null;
+      this.userSpecifiedReceiverSettlementMode = null;
    }
 
    /**
-    * Close the receiver, a closed receiver will throw exceptions if any further send
-    * calls are made.
+    * Close the receiver, a closed receiver will throw exceptions if any further send calls are
+    * made.
     *
-    * @throws IOException if an error occurs while closing the receiver.
+    * @throws IOException
+    *         if an error occurs while closing the receiver.
     */
    public void close() throws IOException {
       if (closed.compareAndSet(false, true)) {
@@ -141,11 +173,20 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       }
    }
 
+   public void setProperties(Map<Symbol, Object> properties) {
+      if (getEndpoint() != null) {
+         throw new IllegalStateException("Endpoint already established");
+      }
+
+      this.properties = properties;
+   }
+
    /**
-    * Detach the receiver, a closed receiver will throw exceptions if any further send
-    * calls are made.
+    * Detach the receiver, a closed receiver will throw exceptions if any further send calls are
+    * made.
     *
-    * @throws IOException if an error occurs while closing the receiver.
+    * @throws IOException
+    *         if an error occurs while closing the receiver.
     */
    public void detach() throws IOException {
       if (closed.compareAndSet(false, true)) {
@@ -179,11 +220,12 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    }
 
    /**
-    * Attempts to wait on a message to be delivered to this receiver.  The receive
-    * call will wait indefinitely for a message to be delivered.
+    * Attempts to wait on a message to be delivered to this receiver. The receive call will wait
+    * indefinitely for a message to be delivered.
     *
     * @return a newly received message sent to this receiver.
-    * @throws Exception if an error occurs during the receive attempt.
+    * @throws Exception
+    *         if an error occurs during the receive attempt.
     */
    public AmqpMessage receive() throws Exception {
       checkClosed();
@@ -191,13 +233,16 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    }
 
    /**
-    * Attempts to receive a message sent to this receiver, waiting for the given
-    * timeout value before giving up and returning null.
+    * Attempts to receive a message sent to this receiver, waiting for the given timeout value
+    * before giving up and returning null.
     *
-    * @param timeout the time to wait for a new message to arrive.
-    * @param unit    the unit of time that the timeout value represents.
+    * @param timeout
+    *        the time to wait for a new message to arrive.
+    * @param unit
+    *        the unit of time that the timeout value represents.
     * @return a newly received message or null if the time to wait period expires.
-    * @throws Exception if an error occurs during the receive attempt.
+    * @throws Exception
+    *         if an error occurs during the receive attempt.
     */
    public AmqpMessage receive(long timeout, TimeUnit unit) throws Exception {
       checkClosed();
@@ -205,11 +250,12 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    }
 
    /**
-    * If a message is already available in this receiver's prefetch buffer then
-    * it is returned immediately otherwise this methods return null without waiting.
+    * If a message is already available in this receiver's prefetch buffer then it is returned
+    * immediately otherwise this methods return null without waiting.
     *
     * @return a newly received message or null if there is no currently available message.
-    * @throws Exception if an error occurs during the receive attempt.
+    * @throws Exception
+    *         if an error occurs during the receive attempt.
     */
    public AmqpMessage receiveNoWait() throws Exception {
       checkClosed();
@@ -220,7 +266,8 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * Request a remote peer send a Message to this client waiting until one arrives.
     *
     * @return the pulled AmqpMessage or null if none was pulled from the remote.
-    * @throws IOException if an error occurs
+    * @throws IOException
+    *         if an error occurs
     */
    public AmqpMessage pull() throws IOException {
       return pull(-1, TimeUnit.MILLISECONDS);
@@ -230,7 +277,8 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * Request a remote peer send a Message to this client using an immediate drain request.
     *
     * @return the pulled AmqpMessage or null if none was pulled from the remote.
-    * @throws IOException if an error occurs
+    * @throws IOException
+    *         if an error occurs
     */
    public AmqpMessage pullImmediate() throws IOException {
       return pull(0, TimeUnit.MILLISECONDS);
@@ -245,10 +293,13 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     *
     * The timeout value when positive is given in milliseconds.
     *
-    * @param timeout the amount of time to tell the remote peer to keep this pull request valid.
-    * @param unit    the unit of measure that the timeout represents.
+    * @param timeout
+    *        the amount of time to tell the remote peer to keep this pull request valid.
+    * @param unit
+    *        the unit of measure that the timeout represents.
     * @return the pulled AmqpMessage or null if none was pulled from the remote.
-    * @throws IOException if an error occurs
+    * @throws IOException
+    *         if an error occurs
     */
    public AmqpMessage pull(final long timeout, final TimeUnit unit) throws IOException {
       checkClosed();
@@ -314,10 +365,26 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Controls the amount of credit given to the receiver link.
     *
-    * @param credit the amount of credit to grant.
-    * @throws IOException if an error occurs while sending the flow.
+    * @param credit
+    *        the amount of credit to grant.
+    * @throws IOException
+    *         if an error occurs while sending the flow.
     */
    public void flow(final int credit) throws IOException {
+      flow(credit, false);
+   }
+
+   /**
+    * Controls the amount of credit given to the receiver link.
+    *
+    * @param credit
+    *        the amount of credit to grant.
+    * @param deferWrite
+    *        defer writing to the wire, hold until for the next operation writes.
+    * @throws IOException
+    *         if an error occurs while sending the flow.
+    */
+   public void flow(final int credit, final boolean deferWrite) throws IOException {
       checkClosed();
       final ClientFuture request = new ClientFuture();
       session.getScheduler().execute(new Runnable() {
@@ -327,7 +394,9 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
             checkClosed();
             try {
                getEndpoint().flow(credit);
-               session.pumpToProtonTransport(request);
+               if (!deferWrite) {
+                  session.pumpToProtonTransport(request);
+               }
                request.onSuccess();
             } catch (Exception e) {
                request.onFailure(e);
@@ -341,8 +410,10 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Attempts to drain a given amount of credit from the link.
     *
-    * @param credit the amount of credit to drain.
-    * @throws IOException if an error occurs while sending the drain.
+    * @param credit
+    *        the amount of credit to drain.
+    * @throws IOException
+    *         if an error occurs while sending the drain.
     */
    public void drain(final int credit) throws IOException {
       checkClosed();
@@ -368,7 +439,8 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Stops the receiver, using all link credit and waiting for in-flight messages to arrive.
     *
-    * @throws IOException if an error occurs while sending the drain.
+    * @throws IOException
+    *         if an error occurs while sending the drain.
     */
    public void stop() throws IOException {
       checkClosed();
@@ -391,12 +463,14 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    }
 
    /**
-    * Accepts a message that was dispatched under the given Delivery instance and settles the delivery.
+    * Accepts a message that was dispatched under the given Delivery instance and settles the
+    * delivery.
     *
     * @param delivery
     *        the Delivery instance to accept.
     *
-    * @throws IOException if an error occurs while sending the accept.
+    * @throws IOException
+    *         if an error occurs while sending the accept.
     */
    public void accept(Delivery delivery) throws IOException {
       accept(delivery, this.session, true);
@@ -410,25 +484,28 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * @param settle
     *        true if the receiver should settle the delivery or just send the disposition.
     *
-    * @throws IOException if an error occurs while sending the accept.
+    * @throws IOException
+    *         if an error occurs while sending the accept.
     */
    public void accept(Delivery delivery, boolean settle) throws IOException {
       accept(delivery, this.session, settle);
    }
 
    /**
-    * Accepts a message that was dispatched under the given Delivery instance and settles the delivery.
+    * Accepts a message that was dispatched under the given Delivery instance and settles the
+    * delivery.
     *
     * This method allows for the session that is used in the accept to be specified by the
-    * caller.  This allows for an accepted message to be involved in a transaction that is
-    * being managed by some other session other than the one that created this receiver.
+    * caller. This allows for an accepted message to be involved in a transaction that is being
+    * managed by some other session other than the one that created this receiver.
     *
     * @param delivery
     *        the Delivery instance to accept.
     * @param session
     *        the session under which the message is being accepted.
     *
-    * @throws IOException if an error occurs while sending the accept.
+    * @throws IOException
+    *         if an error occurs while sending the accept.
     */
    public void accept(final Delivery delivery, final AmqpSession session) throws IOException {
       accept(delivery, session, true);
@@ -438,8 +515,8 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * Accepts a message that was dispatched under the given Delivery instance.
     *
     * This method allows for the session that is used in the accept to be specified by the
-    * caller.  This allows for an accepted message to be involved in a transaction that is
-    * being managed by some other session other than the one that created this receiver.
+    * caller. This allows for an accepted message to be involved in a transaction that is being
+    * managed by some other session other than the one that created this receiver.
     *
     * @param delivery
     *        the Delivery instance to accept.
@@ -448,7 +525,8 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * @param settle
     *        true if the receiver should settle the delivery or just send the disposition.
     *
-    * @throws IOException if an error occurs while sending the accept.
+    * @throws IOException
+    *         if an error occurs while sending the accept.
     */
    public void accept(final Delivery delivery, final AmqpSession session, final boolean settle) throws IOException {
       checkClosed();
@@ -504,14 +582,16 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Mark a message that was dispatched under the given Delivery instance as Modified.
     *
-    * @param delivery the Delivery instance to mark modified.
-    * @param deliveryFailed indicates that the delivery failed for some reason.
-    * @param undeliverableHere marks the delivery as not being able to be process by link it was sent to.
-    * @throws IOException if an error occurs while sending the reject.
+    * @param delivery
+    *        the Delivery instance to mark modified.
+    * @param deliveryFailed
+    *        indicates that the delivery failed for some reason.
+    * @param undeliverableHere
+    *        marks the delivery as not being able to be process by link it was sent to.
+    * @throws IOException
+    *         if an error occurs while sending the reject.
     */
-   public void modified(final Delivery delivery,
-                        final Boolean deliveryFailed,
-                        final Boolean undeliverableHere) throws IOException {
+   public void modified(final Delivery delivery, final Boolean deliveryFailed, final Boolean undeliverableHere) throws IOException {
       checkClosed();
 
       if (delivery == null) {
@@ -546,8 +626,10 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Release a message that was dispatched under the given Delivery instance.
     *
-    * @param delivery the Delivery instance to release.
-    * @throws IOException if an error occurs while sending the release.
+    * @param delivery
+    *        the Delivery instance to release.
+    * @throws IOException
+    *         if an error occurs while sending the release.
     */
    public void release(final Delivery delivery) throws IOException {
       checkClosed();
@@ -581,8 +663,10 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    /**
     * Reject a message that was dispatched under the given Delivery instance.
     *
-    * @param delivery the Delivery instance to reject.
-    * @throws IOException if an error occurs while sending the release.
+    * @param delivery
+    *        the Delivery instance to reject.
+    * @throws IOException
+    *         if an error occurs while sending the release.
     */
    public void reject(final Delivery delivery) throws IOException {
       checkClosed();
@@ -617,10 +701,10 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
     * @return an unmodifiable view of the underlying Receiver instance.
     */
    public Receiver getReceiver() {
-      return new UnmodifiableReceiver(getEndpoint());
+      return UnmodifiableProxy.receiverProxy(getEndpoint());
    }
 
-   //----- Receiver configuration properties --------------------------------//
+   // ----- Receiver configuration properties --------------------------------//
 
    public boolean isPresettle() {
       return presettle;
@@ -662,7 +746,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       return session.getConnection().getDrainTimeout();
    }
 
-   //----- Internal implementation ------------------------------------------//
+   // ----- Internal implementation ------------------------------------------//
 
    @Override
    protected void doOpen() {
@@ -687,13 +771,28 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       Receiver receiver = session.getEndpoint().receiver(receiverName);
       receiver.setSource(source);
       receiver.setTarget(target);
-      if (isPresettle()) {
-         receiver.setSenderSettleMode(SenderSettleMode.SETTLED);
-      } else {
-         receiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
-      }
-      receiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
 
+      if (userSpecifiedSenderSettlementMode != null) {
+         receiver.setSenderSettleMode(userSpecifiedSenderSettlementMode);
+         if (SenderSettleMode.SETTLED.equals(userSpecifiedSenderSettlementMode)) {
+            setPresettle(true);
+         }
+      } else {
+         if (isPresettle()) {
+            receiver.setSenderSettleMode(SenderSettleMode.SETTLED);
+         } else {
+            receiver.setSenderSettleMode(SenderSettleMode.UNSETTLED);
+         }
+      }
+
+      if (userSpecifiedReceiverSettlementMode != null) {
+         receiver.setReceiverSettleMode(userSpecifiedReceiverSettlementMode);
+      } else {
+         receiver.setReceiverSettleMode(ReceiverSettleMode.FIRST);
+      }
+      if (properties != null) {
+         receiver.setProperties(properties);
+      }
       setEndpoint(receiver);
 
       super.doOpen();
@@ -761,7 +860,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
 
    protected void configureSource(Source source) {
       Map<Symbol, DescribedType> filters = new HashMap<>();
-      Symbol[] outcomes = new Symbol[]{Accepted.DESCRIPTOR_SYMBOL, Rejected.DESCRIPTOR_SYMBOL, Released.DESCRIPTOR_SYMBOL, Modified.DESCRIPTOR_SYMBOL};
+      Symbol[] outcomes = new Symbol[] {Accepted.DESCRIPTOR_SYMBOL, Rejected.DESCRIPTOR_SYMBOL, Released.DESCRIPTOR_SYMBOL, Modified.DESCRIPTOR_SYMBOL};
 
       if (getSubscriptionName() != null && !getSubscriptionName().isEmpty()) {
          source.setExpiryPolicy(TerminusExpiryPolicy.NEVER);
@@ -794,7 +893,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    }
 
    @Override
-   public void processDeliveryUpdates(AmqpConnection connection) throws IOException {
+   public void processDeliveryUpdates(AmqpConnection connection, Delivery delivery) throws IOException {
       Delivery incoming = null;
       do {
          incoming = getEndpoint().current();
@@ -821,12 +920,15 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
                }
             }
          }
-      } while (incoming != null);
+      }
+      while (incoming != null);
 
-      super.processDeliveryUpdates(connection);
+      super.processDeliveryUpdates(connection, delivery);
    }
 
    private void processDelivery(Delivery incoming) throws Exception {
+      doDeliveryInspection(incoming);
+
       Message message = null;
       try {
          message = decodeIncomingMessage(incoming);
@@ -849,6 +951,14 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       }
    }
 
+   private void doDeliveryInspection(Delivery delivery) {
+      try {
+         getStateInspector().inspectDelivery(getReceiver(), delivery);
+      } catch (Throwable error) {
+         getStateInspector().markAsInvalid(error.getMessage());
+      }
+   }
+
    @Override
    public void processFlowUpdates(AmqpConnection connection) throws IOException {
       if (pullRequest != null || stopRequest != null) {
@@ -932,7 +1042,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
                @Override
                public void run() {
                   LOG.trace("Consumer {} drain request timed out", this);
-                  Exception cause = new JmsOperationTimedOutException("Remote did not respond to a drain request in time");
+                  Exception cause = new AmqpOperationTimedOutException("Remote did not respond to a drain request in time");
                   locallyClosed(session.getConnection(), cause);
                   stopRequest.onFailure(cause);
                   session.pumpToProtonTransport(stopRequest);
@@ -972,7 +1082,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
       }
    }
 
-   //----- Internal Transaction state callbacks -----------------------------//
+   // ----- Internal Transaction state callbacks -----------------------------//
 
    void preCommit() {
    }
@@ -986,7 +1096,7 @@ public class AmqpReceiver extends AmqpAbstractResource<Receiver> {
    void postRollback() {
    }
 
-   //----- Inner classes used in message pull operations --------------------//
+   // ----- Inner classes used in message pull operations --------------------//
 
    protected static final class ScheduledRequest implements AsyncResult {
 
