@@ -357,6 +357,8 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
 
    private final Long id;
 
+   private boolean durable;
+
    private long messageCount;
 
    public FakeQueue(final SimpleString name) {
@@ -394,7 +396,7 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
    }
 
    @Override
-   public void acknowledge(Transaction tx, MessageReference ref, AckReason reason, ServerConsumer consumer) throws Exception {
+   public void acknowledge(Transaction tx, MessageReference ref, AckReason reason, ServerConsumer consumer, boolean decDel) throws Exception {
       // no-op
 
    }
@@ -490,7 +492,7 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
    }
 
    @Override
-   public void expire(final MessageReference ref, final ServerConsumer consumer) throws Exception {
+   public void expire(final MessageReference ref, final ServerConsumer consumer, boolean decDel) throws Exception {
       // no-op
 
    }
@@ -719,7 +721,12 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
    @Override
    public boolean isDurableMessage() {
       // no-op
-      return false;
+      return durable;
+   }
+
+   public FakeQueue setDurable(boolean durable) {
+      this.durable = durable;
+      return this;
    }
 
    @Override
@@ -889,11 +896,12 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
 
    }
 
-   public void setPageSubscription(PageSubscription sub) {
+   public FakeQueue setPageSubscription(PageSubscription sub) {
       this.subs = sub;
       if (subs != null) {
          sub.setQueue(this);
       }
+      return this;
    }
 
    @Override
@@ -958,6 +966,10 @@ public class FakeQueue extends CriticalComponentImpl implements Queue {
       return null;
    }
 
+   @Override
+   public void postAcknowledge(MessageReference ref, AckReason reason, boolean delivering) {
+   }
+
    @Override
    public void postAcknowledge(MessageReference ref, AckReason reason) {
    }
