@@ -31,7 +31,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 
 import io.netty.util.collection.LongObjectHashMap;
 import org.apache.activemq.artemis.tests.util.ActiveMQTestBase;
-import org.apache.activemq.artemis.tests.util.RandomUtil;
+import org.apache.activemq.artemis.utils.RandomUtil;
 import org.apache.activemq.artemis.utils.collections.NodeStore;
 import org.apache.activemq.artemis.utils.collections.LinkedListImpl;
 import org.apache.activemq.artemis.utils.collections.LinkedListIterator;
@@ -1032,6 +1032,41 @@ public class LinkedListTest extends ActiveMQTestBase {
 
    }
 
+   @Test
+   public void testRemoveLastNudgeNoReplay() {
+      for (int i = 1; i < 3; i++) {
+         doTestRemoveLastNudgeNoReplay(i);
+      }
+   }
+
+   private void doTestRemoveLastNudgeNoReplay(int num) {
+
+      LinkedListIterator<Integer> iter = list.iterator();
+
+      for (int i = 0; i < num; i++) {
+         list.addTail(i);
+      }
+
+      // exhaust iterator
+      for (int i = 0; i < num; i++) {
+         assertTrue(iter.hasNext());
+         assertEquals(i, iter.next().intValue());
+      }
+
+      // remove last
+      LinkedListIterator<Integer> pruneIterator = list.iterator();
+      while (pruneIterator.hasNext()) {
+         int v = pruneIterator.next();
+         if (v == num - 1) {
+            pruneIterator.remove();
+         }
+      }
+
+      // ensure existing iterator does not reset or replay
+      assertFalse(iter.hasNext());
+      assertEquals(num - 1, list.size());
+   }
+
    @Test
    public void testGCNepotismPoll() {
       final int count = 100;
@@ -1385,6 +1420,29 @@ public class LinkedListTest extends ActiveMQTestBase {
 
    }
 
+
+   @Test
+   public void testGetElement() {
+
+      for (int i = 0; i < 100; i++) {
+         list.addTail(i);
+      }
+
+      for (int i = 0; i < 100; i++) {
+         Assert.assertEquals(i, list.get(i).intValue());
+      }
+
+      boolean expectedException = false;
+
+      try {
+         list.get(100);
+      } catch (IndexOutOfBoundsException e) {
+         expectedException = true;
+      }
+
+      Assert.assertTrue(expectedException);
+   }
+
    @Test
    public void testResizing() {
       int numIters = 1000;
